#                                                       #
#                   Test PloneMaintenance               #
#                                                       #
# (c)2002 Ingeniweb                                     #
#                                                       #

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# use recent version (0.6+) of PloneTestCase from CVS:
from Products.PloneTestCase import PloneTestCase

from Products.PloneMaintenance.MaintenanceTask import addMaintenanceTask
from Products.PloneMaintenance.global_symbols import *
from Products.PloneMaintenance.PloneMaintenanceTool import getNoSecDate

PloneTestCase.installProduct('PloneMaintenance')
PloneTestCase.setupPloneSite()

from AccessControl import getSecurityManager

from DateTime import DateTime

# Set log options if Log module is available
# This is done to set LOG_PROCESSORs to file logs instead of Zope logs
try:
    import Log

    Log.LOG_LEVEL = Log.LOG_DEBUG

    Log.LOG_PROCESSOR = {
        Log.LOG_NONE: Log.logFile,
        Log.LOG_CRITICAL: Log.logFile,
        Log.LOG_ERROR: Log.logFile,
        Log.LOG_WARNING: Log.logFile,
        Log.LOG_NOTICE: Log.logFile,
        Log.LOG_DEBUG: Log.logFile,
        }

    from Log import *
    Log(LOG_NOTICE, "Starting %s at %d debug level" % (os.path.dirname(__file__), LOG_LEVEL, ))

except:
    print "Log module not available"
    LOG_DEBUG = None
    LOG_NOTICE = None
    LOG_WARNING = None
    LOG_ERROR = None
    LOG_CRITICAL = None
    def Log(*args, **kw):
        pass
    raise

class TestPloneMaintenance(PloneTestCase.PloneTestCase):

    def afterSetUp(self):
        self.loginAsPortalOwner()
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct('PloneMaintenance')
        self.securityManager = getSecurityManager()

    def login(self, userid=None):
        if userid:
            PloneTestCase.PloneTestCase.login(self, userid)
            pm = getToolByName(self.portal, 'portal_membership')
            self.portal.REQUEST.set('_portaluser', pm.getMemberById(userid))
        else:
            PloneTestCase.PloneTestCase.login(self)

    def test_01_check_defaults(self,):
        """
        """
        # Set a valid model
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        self.failUnless(task.getProperty('scheduled_month')=='*')
        #self.failUnless(task.getProperty('scheduled_day_of_week')=='*')
        self.failUnless(task.getProperty('scheduled_day_of_month')=='*')
        self.failUnless(task.getProperty('scheduled_hour')=='*')
        self.failUnless(task.getProperty('scheduled_minute')=='*')
        self.failUnless(task.getProperty('script_name')=='')
        date = getNoSecDate(DateTime())
        self.failUnless(task.isPending(date)==1) # By default always run.
        current_date=getNoSecDate(DateTime())
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(next_scheduled >= date, str(next_scheduled) + " not >= " + str(date))


    def test_02_isPending(self,):
        """
        """
        # Set a valid model
        current_date=getNoSecDate(DateTime())
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)

        # Should be run this month
        task._updateProperty('scheduled_month',str(current_date.month()))

        next_scheduled = task.getNextScheduledExecutionTime(current_date)

        self.failUnless( next_scheduled >= current_date, str(next_scheduled) + " not >= " + str(current_date))

    def test_03_execution(self,):
        """
        """
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        date = getNoSecDate(DateTime())
        self.failUnless(task.runTask(date)==0) # Didn't run as script is empty

    def test_04_beforeIntervall(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/05/16 15:40')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_hour','15')
        task._updateProperty('scheduled_minute','20')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(DateTime('2004/05/17 15:20') == next_scheduled, str(DateTime('2004/05/17 15:20')) + ' != ' + str(next_scheduled))


    def test_05_beforeIntervall(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/05/01 00:40')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','05')
        task._updateProperty('scheduled_minute','10')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(DateTime('2005/05/01 00:10')==next_scheduled)

    def test_06_beforeIntervall(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/05/01 00:40')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(DateTime('2004/05/01 00:40')==next_scheduled)

    def test_07_beforeIntervall(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/05/17 10:16')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','05')
        task._updateProperty('scheduled_day_of_month','17')
        task._updateProperty('scheduled_hour','10')
        task._updateProperty('scheduled_minute','15')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(DateTime('2005/05/17 10:15')==next_scheduled)

    def test_08_nextYear(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/05/17 10:16')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','04')
        task._updateProperty('scheduled_day_of_month','*')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','*')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(DateTime('2005/04/01 00:00') == next_scheduled)

    def test_09_nextMonth(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/05/17 10:16')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','04')
        task._updateProperty('scheduled_day_of_month','1')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','*')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(DateTime('2005/04/01 00:00') == next_scheduled)

    def test_10_month_end(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/02/29 10:16')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','02')
        task._updateProperty('scheduled_day_of_month','28')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','*')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(DateTime('2005/28/02 00:00') == next_scheduled, str(DateTime('2005/28/02 00:00')) + ' != ' + str(next_scheduled))

    def test_11_minute(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','*')
        task._updateProperty('scheduled_day_of_month','*')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','47')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/04/25 09:47')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_12_hour(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','*')
        task._updateProperty('scheduled_day_of_month','*')
        task._updateProperty('scheduled_hour','10')
        task._updateProperty('scheduled_minute','*')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(DateTime('2004/04/25 10:00') == next_scheduled)

    def test_13_day(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','*')
        task._updateProperty('scheduled_day_of_month','27')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','*')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(DateTime('2004/04/27 00:00') == next_scheduled)

    def test_14_month(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','12')
        task._updateProperty('scheduled_day_of_month','*')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','*')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        self.failUnless(DateTime('2004/12/01 00:00') == next_scheduled, str(DateTime('2004/12/01 00:00'))+' != ' + str(next_scheduled))

    def test_15_day_minute(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','*')
        task._updateProperty('scheduled_day_of_month','26')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','12')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/04/26 00:12')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_16_day_hour(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_day_of_month','26')
        task._updateProperty('scheduled_hour','11')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/04/26 11:00')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_17_day_hour_minute(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_day_of_month','26')
        task._updateProperty('scheduled_hour','11')
        task._updateProperty('scheduled_minute','11')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/04/26 11:11')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_18_month_hour(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','07')
        task._updateProperty('scheduled_hour','11')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/07/01 11:00')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_19_month_hour_minute(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','07')
        task._updateProperty('scheduled_hour','11')
        task._updateProperty('scheduled_minute','11')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/07/01 11:11')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_20_month_day_minute(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','07')
        task._updateProperty('scheduled_day_of_month','11')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','11')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/07/11 00:11')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))


    def test_21_month_day_hour(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:45')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','07')
        task._updateProperty('scheduled_day_of_month','11')
        task._updateProperty('scheduled_hour','11')
        task._updateProperty('scheduled_minute','*')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/07/11 11:00')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_22_minute(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:47')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','*')
        task._updateProperty('scheduled_day_of_month','*')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','47')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/04/25 09:47')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_23_minute(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 09:49')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','*')
        task._updateProperty('scheduled_day_of_month','*')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','47')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/04/25 10:47')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_24_minute(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 23:49')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','*')
        task._updateProperty('scheduled_day_of_month','*')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','47')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/04/26 00:47')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_25_minute(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 23:49')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','*')
        task._updateProperty('scheduled_day_of_month','*')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','47')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/04/26 00:47')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))

    def test_26_minute_day(self,):
        """
        """
        # Set a valid model
        current_date=DateTime('2004/04/25 23:49')
        content_id = "task1"
        addMaintenanceTask(self.portal, content_id, title = 'Task 1')
        task=getattr(self.portal, content_id)
        task._updateProperty('scheduled_month','*')
        task._updateProperty('scheduled_day_of_month','07')
        task._updateProperty('scheduled_hour','*')
        task._updateProperty('scheduled_minute','47')
        next_scheduled = task.getNextScheduledExecutionTime(current_date)
        correct_date = DateTime('2004/05/07 00:47')
        self.failUnless(correct_date == next_scheduled, str(correct_date) +' != ' + str(next_scheduled))


if __name__ == '__main__':
    framework(descriptions=1, verbosity=1)
else:
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(TestPloneMaintenance))
        return suite

